@@ -1,5 +1,5 @@
 /*
- * Copyright © 2009 inria.  All rights reserved.
+ * Copyright © 2009-2016 Inria.  All rights reserved.
  * Copyright © 2009 Université Bordeaux
  * See COPYING in top-level directory.
  */
@@ -14,4 +14,10 @@ typedef int processorid_t;
 
 extern int processor_bind(idtype_t idtype, id_t id, processorid_t processorid, processorid_t *obind);
 
+/* hide Linux' host disabling _SC_LARGE_PAGESIZE */
+#undef HAVE_DECL__SC_LARGE_PAGESIZE
+#define HAVE_DECL__SC_LARGE_PAGESIZE 1
+#undef _SC_LARGE_PAGESIZE
+#define _SC_LARGE_PAGESIZE 33
+
 #endif /* HWLOC_PORT_SOLARIS_SYS_PROCESSOR_H */
